@@ -1,6 +1,6 @@
 # Software License Agreement (BSD License)
 #
-# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
+# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
 # All rights reserved.
 #
 # Redistribution and use of this software in source and binary forms, with or
@@ -12,7 +12,7 @@
 #   following disclaimer.
 #
 #   Redistributions in binary form must reproduce the above
-#   copyright notice, this list of conditions and the
+#   copyright notice, this list of conditions and the   
 #   following disclaimer in the documentation and/or other
 #   materials provided with the distribution.
 #
@@ -30,16 +30,21 @@
 #
 # Author: vic.iglesias@eucalyptus.com
 # Author: matt.clark@eucalyptus.com
+
 from boto.ec2.image import Image
 from boto.ec2.volume import Volume
 from cwops import CWops
 from asops import ASops
+from eucaops.cfnops import CFNops
 from eucaops.elbops import ELBops
 from iamops import IAMops
 from ec2ops import EC2ops
 from s3ops import S3ops
 from stsops import STSops
 import time
+import traceback
+import sys
+import StringIO
 from eutester.euservice import EuserviceManager
 from boto.ec2.instance import Reservation
 from boto.exception import EC2ResponseError
@@ -51,12 +56,14 @@ from eutester import eulogger
 import re
 import os
 
-class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
+class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops, CFNops):
     
     def __init__(self, config_file=None, password=None, keypath=None, credpath=None, aws_access_key_id=None,
                  aws_secret_access_key = None,  account="eucalyptus", user="admin", username=None, APIVersion='2011-01-01',
-                 region=None, ec2_ip=None, s3_ip=None, as_ip=None, elb_ip=None, download_creds=True,boto_debug=0,
-                 debug_method=None):
+                 ec2_ip=None, ec2_path=None, iam_ip=None, iam_path=None, s3_ip=None, s3_path=None,
+                 as_ip=None, as_path=None, elb_ip=None, elb_path=None, cw_ip=None, cw_path=None,
+                 cfn_ip=None, cfn_path=None, sts_ip=None, sts_path=None,
+                 port=8773, download_creds=True, boto_debug=0, debug_method=None, region=None):
         self.config_file = config_file 
         self.APIVersion = APIVersion
         self.eucapath = "/opt/eucalyptus"
@@ -73,6 +80,8 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
         self.key_dir = "./"
         self.clc_index = 0
         self.credpath = credpath
+        self.account_name = account
+        self.aws_username = user
         self.download_creds = download_creds
         self.logger = eulogger.Eulogger(identifier="EUCAOPS")
         self.debug = debug_method or self.logger.log.debug
@@ -82,6 +91,7 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
         self.account_id = None
         self.aws_access_key_id = aws_access_key_id
         self.aws_secret_access_key = aws_secret_access_key
+        self._property_manager = None
 
 
         if self.config_file is not None:
@@ -102,8 +112,6 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
             ### Need to create service manager for user if we have an ssh connection and password
             clc_array = self.get_component_machines("clc")
             self.clc = clc_array[0]
-            walrus_array = self.get_component_machines("ws")
-            self.walrus = walrus_array[0]
             self.sftp = self.clc.ssh.connection.open_sftp()
             if self.download_creds:
                 if self.credpath is None:
@@ -121,13 +129,10 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
                             raise Exception(str(tb) + "\nCould not get credentials from first CLC and no other to try")
                         self.swap_clc()
                         self.sftp = self.clc.ssh.connection.open_sftp()
-                        self.get_credentials(account,user)
+                        self.get_credentials(account, user)
                         
                 self.service_manager = EuserviceManager(self)
                 self.clc = self.service_manager.get_enabled_clc().machine
-                self.walrus = self.service_manager.get_enabled_walrus().machine 
-
-
 
         if self.credpath and not aws_access_key_id:
             aws_access_key_id = self.get_access_key()
@@ -138,11 +143,39 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
             try:
                 if self.credpath and not ec2_ip:
                     ec2_ip = self.get_ec2_ip()
-                self.setup_ec2_connection(endpoint=ec2_ip, path="/services/Eucalyptus", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, APIVersion=APIVersion, boto_debug=boto_debug)
+                if self.credpath and not ec2_path:
+                    ec2_path = self.get_ec2_path()
+
+                self.setup_ec2_connection(endpoint=ec2_ip, path=ec2_path, port=port, is_secure=False,
+                                          region=region, aws_access_key_id=aws_access_key_id,
+                                          aws_secret_access_key=aws_secret_access_key, APIVersion=APIVersion,
+                                          boto_debug=boto_debug)
                 self.setup_ec2_resource_trackers()
-                self.setup_iam_connection(endpoint=ec2_ip, path="/services/Euare", port=8773, is_secure=False, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,  boto_debug=boto_debug)
-                self.setup_sts_connection( endpoint=ec2_ip, path="/services/Eucalyptus", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
-                self.setup_cw_connection( endpoint=ec2_ip, path="/services/CloudWatch", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+
+                if self.credpath and not iam_ip:
+                    iam_ip = self.get_iam_ip()
+                if self.credpath and not iam_path:
+                    iam_path = self.get_iam_path()
+                self.setup_iam_connection(endpoint=iam_ip, path=iam_path,
+                                          port=port, is_secure=False, aws_access_key_id=aws_access_key_id,
+                                          aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+
+                if self.credpath and not sts_ip:
+                    sts_ip = self.get_sts_ip()
+                if self.credpath and not sts_path:
+                    sts_path = self.get_sts_path()
+                self.setup_sts_connection(endpoint=sts_ip, path=sts_path, port=port, is_secure=False,
+                                          region=region, aws_access_key_id=aws_access_key_id,
+                                          aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+
+                if self.credpath and not cw_ip:
+                    cw_ip = self.get_cw_ip()
+                if self.credpath and not cw_path:
+                    cw_path = self.get_cw_path()
+                self.setup_cw_connection(endpoint=cw_ip, path=cw_path, port=port, is_secure=False,
+                                         region=region, aws_access_key_id=aws_access_key_id,
+                                         aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+
                 self.setup_cw_resource_trackers()
             except Exception, e:
                 tb = self.get_traceback()
@@ -151,30 +184,68 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
             try:
                 if self.credpath and not s3_ip:
                     s3_ip = self.get_s3_ip()
-                self.setup_s3_connection(endpoint=s3_ip, path="/services/Walrus", port=8773, is_secure=False,aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,  boto_debug=boto_debug)
+                if self.credpath and not s3_path:
+                    s3_path = self.get_s3_path()
+                self.setup_s3_connection(endpoint=s3_ip, path=s3_path, port=port, is_secure=False,
+                                         aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,
+                                         boto_debug=boto_debug)
                 self.setup_s3_resource_trackers()
             except Exception, e:
-                raise Exception("Unable to create S3 connection because of: " + str(e) )
+                self.debug("Unable to create S3 connection because of: " + str(e) )
 
             try:
                 if self.credpath and not as_ip:
                     as_ip = self.get_as_ip()
-                self.setup_as_connection(endpoint=as_ip, path="/services/AutoScaling", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+                if self.credpath and not as_path:
+                    as_path = self.get_as_path()
+                self.setup_as_connection(endpoint=as_ip, path=as_path, port=port, is_secure=False,
+                                         region=region, aws_access_key_id=aws_access_key_id,
+                                         aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
             except Exception, e:
                 self.debug("Unable to create AS connection because of: " + str(e) )
 
             try:
                 if self.credpath and not elb_ip:
                     elb_ip = self.get_elb_ip()
-                self.setup_elb_connection(endpoint=elb_ip, path="/services/LoadBalancing", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+                if self.credpath and not elb_path:
+                    elb_path = self.get_elb_path()
+                self.setup_elb_connection(endpoint=elb_ip, path=elb_path, port=port, is_secure=False,
+                                          region=region, aws_access_key_id=aws_access_key_id,
+                                          aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
             except Exception, e:
                 self.debug("Unable to create ELB connection because of: " + str(e) )
-        if self.clc and account == 'eucalytpus':
-            self.update_property_manager()
+            try:
+                if self.credpath and not cfn_ip:
+                    cfn_ip = self.get_cfn_ip()
+                if self.credpath and not cfn_path:
+                    cfn_path = self.get_cfn_path()
+                self.setup_cfn_connection(endpoint=cfn_ip, path=self.get_cfn_path(), port=port, is_secure=False,
+                                          region=region, aws_access_key_id=aws_access_key_id,
+                                          aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
+            except Exception, e:
+                self.debug("Unable to create CloudFormation connection because of: " + str(e) )
+
+
+
+    @property
+    def property_manager(self):
+        if not self._property_manager:
+            if self.clc and self.account_name == 'eucalyptus':
+                try:
+                    self.update_property_manager()
+                except:
+                    tb = self.get_traceback()
+                    self.debug(str(tb) + '\nError creating properties manager')
+        return self._property_manager
+
+
+    @property_manager.setter
+    def property_manager(self, new_property_mgr):
+        self._property_manager = new_property_mgr
 
     def get_available_vms(self, type=None, zone=None):
         """
-        Get available VMs of a certain type or return a dictionary with all types and their available vms
+        Get available VMs of a certain type, defaults to m1.small
         type        VM type to get available vms 
         """
         
@@ -227,31 +298,59 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
         property        Property to modify
         value           Value to set it too
         """
+        if self.credpath == None or self.eucapath == None or property == None or value == None:
+            self.fail("Cannot set property value due to insufficient arguments")
+            self.fail("credpath: " + ("None" if self.credpath is None else self.credpath))
+            self.fail("eucapath: " + ("None" if self.eucapath is None else self.eucapath))
+            self.fail("property: " + ("None" if property is None else property))
+            self.fail("value: " + ("None" if value is None else value))
+            raise Exception("Cannot set property: " + (property if property is not None else "unknown"))
+        
         command = "source " + self.credpath + "/eucarc && " + self.eucapath + "/usr/sbin/euca-modify-property -p " + property + "=" + value
         if self.clc.found(command, property):
             self.debug("Properly modified property " + property)
         else:
             raise Exception("Setting property " + property + " failed")
-    
-   
-    def cleanup_artifacts(self,instances=True, snapshots=True, volumes=True, load_balancers=True):
+
+
+    def cleanup_artifacts(self,instances=True, snapshots=True, volumes=True,
+                          load_balancers=True, ip_addresses=True,
+                          auto_scaling_groups=True, launch_configurations=True,
+                          keypairs=True):
         """
         Description: Attempts to remove artifacts created during and through this eutester's lifespan.
         """
         failmsg = ""
         failcount = 0
         self.debug("Starting cleanup of artifacts")
+        if auto_scaling_groups:
+            try:
+                self.cleanup_autoscaling_groups()
+            except Exception, e:
+                tb = self.get_traceback()
+                failcount +=1
+                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
         if instances:
             for res in self.test_resources["reservations"]:
                 try:
                     self.terminate_instances(res)
+                    if res in self.test_resources["reservations"]:
+                        self.test_resources["reservations"].remove(res)
                 except Exception, e:
                     tb = self.get_traceback()
                     failcount +=1
                     failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
+        if ip_addresses:
+            try:
+                self.cleanup_addresses()
+            except Exception, e:
+                tb = self.get_traceback()
+                failcount +=1
+                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
         if volumes:
             try:
                 self.clean_up_test_volumes(timeout_per_vol=60)
+                self.test_resources['volumes']=[]
             except Exception, e:
                 tb = self.get_traceback()
                 failcount +=1
@@ -271,6 +370,14 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
                 failcount +=1
                 failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
 
+        if launch_configurations:
+            try:
+                self.cleanup_launch_configs()
+            except Exception, e:
+                tb = self.get_traceback()
+                failcount +=1
+                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
+
         for key,array in self.test_resources.iteritems():
             for item in array:
                 try:
@@ -278,18 +385,16 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
                     self.debug("Deleting " + str(item))
                     if isinstance(item, Image):
                         item.deregister()
-                    elif isinstance(item, Reservation):
-                        self.terminate_instances(item)
-                    elif isinstance(item, Volume):
-                        try:
-                            self.detach_volume(item)
-                        except:
-                            pass
-                        item.update()
-                        if item.status != 'deleted':
-                            self.delete_volume(item)
+                    elif isinstance(item,Reservation):
+                        continue
                     else:
-                        item.delete()
+                        try:
+                            item.delete()
+                        except EC2ResponseError as ec2re:
+                            if ec2re.status == 400:
+                                self.debug('Resource not found assuming it is'
+                                           ' already deleted, resource:'
+                                           + str(item))
                 except Exception, e:
                     tb = self.get_traceback()
                     failcount += 1
@@ -297,6 +402,14 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
         if failmsg:
             failmsg += "\nFound " + str(failcount) + " number of errors while cleaning up. See above"
             raise Exception(failmsg)
+        if launch_configurations:
+            try:
+                self.cleanup_launch_configs()
+            except Exception, e:
+                tb = self.get_traceback()
+                failcount +=1
+                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
+
 
     def cleanup_load_balancers(self, lbs=None):
         """
@@ -310,6 +423,21 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
             except KeyError:
                 self.debug("No loadbalancers to delete")
 
+    def cleanup_addresses(self, ips=None):
+        """
+        :param ips: optional list of ip addresses, else will attempt to delete from test_resources[]
+
+        """
+        addresses = ips or self.test_resources['addresses']
+        if not addresses:
+            return
+
+        self.debug('Attempting to release to the cloud the following IP addresses:')
+
+        while addresses:
+            self.release_address(addresses.pop())
+
+
     def cleanup_test_snapshots(self,snaps=None, clean_images=False, add_time_per_snap=10, wait_for_valid_state=120, base_timeout=180):
         """
         :param snaps: optional list of snapshots, else will attempt to delete from test_resources[]
@@ -356,8 +484,6 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
             return
         self.debug('clean_up_test_volumes starting\nVolumes to be deleted:' + ",".join(str(x) for x in volumes))
 
-
-
         for vol in volumes:
             try:
                 vol = self.get_volume(volume_id=vol.id)
@@ -365,6 +491,8 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
                 tb = self.get_traceback()
                 self.debug("\n" + line + " Ignoring caught Exception:\n" + str(tb) + "\n"+ str(vol.id) +
                            ', Could not retrieve volume, may no longer exist?' + line)
+                if vol in self.test_resources['volumes']:
+                    self.test_resources['volumes'].remove(vol)
                 vol = None
             if vol:
                 try:
@@ -499,19 +627,24 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
                 machines.append(cloud_machine)
                 
             ### LOOK for network mode in config file if not found then set it unknown
-            try:
-                if re.search("^network",line, re.IGNORECASE):
-                    config_hash["network"] = line.split()[1].lower()
-            except:
-                self.debug("Could not find network type setting to unknown")
-                config_hash["network"] = "unknown"
+            for param in ["network", "managed_ips", "subnet_ip"]:
+                try:
+                    if re.search("^" + param + ".*$", line, re.IGNORECASE):
+                        config_hash[param] = " ".join(line.split()[1:])
+                except:
+                    self.debug("Could not find " + param + " type setting to None")
+                    config_hash[param] = None
         #f.close()   
         config_hash["machines"] = machines 
         return config_hash
 
+
     def update_property_manager(self,machine=None):
         machine = machine or self.clc
-        self.property_manager = Euproperty_Manager(self,debugmethod=self.debug)
+        if not machine:
+            self._property_manager = None
+            return
+        self._property_manager = Euproperty_Manager(self, machine=machine, debugmethod=self.debug)
 
     def swap_clc(self):
         all_clcs = self.get_component_machines("clc")
@@ -525,12 +658,12 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
 
     def swap_walrus(self):
         all_walruses = self.get_component_machines("ws")
-        if self.walrus is all_walruses[0]: 
+        if self.object_storage is all_walruses[0]:
             self.debug("Swapping Walrus from " + all_walruses[0].hostname + " to " + all_walruses[1].hostname)
-            self.walrus = all_walruses[1]
-        elif self.walrus is all_walruses[1]:
+            self.object_storage = all_walruses[1]
+        elif self.object_storage is all_walruses[1]:
             self.debug("Swapping Walrus from " + all_walruses[1].hostname + " to " + all_walruses[0].hostname)
-            self.walrus = all_walruses[0]
+            self.object_storage = all_walruses[0]
             
     def get_network_mode(self):
         return self.config['network']
@@ -628,7 +761,7 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
     def download_creds_from_clc(self, admin_cred_dir):
         self.debug("Downloading credentials from " + self.clc.hostname + ", path:" + admin_cred_dir + "/creds.zip")
         self.sftp.get(admin_cred_dir + "/creds.zip" , admin_cred_dir + "/creds.zip")
-        os.system("unzip -o " + admin_cred_dir + "/creds.zip -d " + admin_cred_dir )
+        self.local("unzip -o " + admin_cred_dir + "/creds.zip -d " + admin_cred_dir )
     
     def send_creds_to_machine(self, admin_cred_dir, machine):
         self.debug("Sending credentials to " + machine.hostname)
@@ -639,7 +772,7 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
             machine.sys("mkdir " + admin_cred_dir)
             machine.sftp.put( admin_cred_dir + "/creds.zip" , admin_cred_dir + "/creds.zip")
             machine.sys("unzip -o " + admin_cred_dir + "/creds.zip -d " + admin_cred_dir )
-            machine.sys("sed -i 's/" + self.get_ec2_ip() + "/" + machine.hostname  +"/g' " + admin_cred_dir + "/eucarc")
+            # machine.sys("sed -i 's/" + self.get_ec2_ip() + "/" + machine.hostname  +"/g' " + admin_cred_dir + "/eucarc")
             
         
     def setup_local_creds_dir(self, admin_cred_dir):
@@ -657,5 +790,17 @@ class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
         """
         return self.clc.sys(cmd, verbose=verbose,  listformat=listformat, timeout=timeout, code=code)
     
-
+    @classmethod
+    def get_traceback(cls):
+        '''
+        Returns a string buffer with traceback, to be used for debug/info purposes. 
+        '''
+        try:
+            out = StringIO.StringIO()
+            traceback.print_exception(*sys.exc_info(),file=out)
+            out.seek(0)
+            buf = out.read()
+        except Exception, e:
+                buf = "Could not get traceback"+str(e)
+        return str(buf) 
     
