@@ -1,6 +1,6 @@
 # Software License Agreement (BSD License)
 #
-# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
+# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
 # All rights reserved.
 #
 # Redistribution and use of this software in source and binary forms, with or
@@ -35,15 +35,19 @@ import boto
 
 class IAMops(Eutester):
 
-    def __init__(self,credpath=None, endpoint="iam.amazonaws.com", aws_access_key_id=None,aws_secret_access_key=None, is_secure=True, port=443, path='/', boto_debug=0 ):
+    def __init__(self,credpath=None, endpoint="iam.amazonaws.com", aws_access_key_id=None, aws_secret_access_key=None,
+                 is_secure=True, port=443, path='/', boto_debug=0 ):
         self.aws_access_key_id = aws_access_key_id
         self.aws_secret_access_key = aws_secret_access_key
         self.user_id = None
         self.account_id = None
         super(IAMops, self).__init__(credpath=credpath)
-        self.setup_iam_connection(endpoint=endpoint, aws_access_key_id=self.aws_access_key_id ,aws_secret_access_key=self.aws_secret_access_key, is_secure=is_secure, port=port,path=path, boto_debug=boto_debug )
+        self.setup_iam_connection(endpoint=endpoint, aws_access_key_id=self.aws_access_key_id ,
+                                  aws_secret_access_key=self.aws_secret_access_key, is_secure=is_secure, port=port,
+                                  path=path, boto_debug=boto_debug )
 
-    def setup_iam_connection(self, endpoint="iam.amazonaws.com", aws_access_key_id=None,aws_secret_access_key=None, is_secure=True, port=443, path='/', boto_debug=0 ):
+    def setup_iam_connection(self, endpoint="iam.amazonaws.com", aws_access_key_id=None,aws_secret_access_key=None,
+                             is_secure=True, port=443, path='/', boto_debug=0 ):
         try:
             euare_connection_args = { 'aws_access_key_id' : aws_access_key_id,
                                       'aws_secret_access_key': aws_secret_access_key,
@@ -56,7 +60,18 @@ class IAMops(Eutester):
             self.euare = boto.connect_iam(**euare_connection_args)
         except Exception, e:
             self.critical("Was unable to create IAM connection because of exception: " + str(e))
-    
+
+    def get_iam_ip(self):
+        """Parse the eucarc for the EUARE_URL"""
+        iam_url = self.parse_eucarc("EUARE_URL")
+        return iam_url.split("/")[2].split(":")[0]
+
+    def get_iam_path(self):
+        """Parse the eucarc for the EUARE_URL"""
+        iam_url = self.parse_eucarc("EUARE_URL")
+        iam_path = "/".join(iam_url.split("/")[3:])
+        return iam_path
+
     def create_account(self,account_name):
         """
         Create an account with the given name
@@ -607,6 +622,41 @@ class IAMops(Eutester):
         access_tuple['access_key_id'] = response['create_access_key_response']['create_access_key_result']['access_key']['access_key_id']
         access_tuple['secret_access_key'] = response['create_access_key_response']['create_access_key_result']['access_key']['secret_access_key']
         return access_tuple
-    
-        
-    
+
+    def upload_server_cert(self, cert_name, cert_body, private_key):
+        self.debug("uploading server certificate: " + cert_name)
+        self.euare.upload_server_cert(cert_name=cert_name, cert_body=cert_body, private_key=private_key)
+        if cert_name not in str(self.euare.get_server_certificate(cert_name)):
+            raise Exception("certificate " + cert_name + " not uploaded")
+
+    def update_server_cert(self, cert_name, new_cert_name=None, new_path=None):
+        self.debug("updating server certificate: " + cert_name)
+        self.euare.update_server_cert(cert_name=cert_name, new_cert_name=new_cert_name, new_path=new_path)
+        if (new_cert_name and new_path) not in str(self.euare.get_server_certificate(new_cert_name)):
+            raise Exception("certificate " + cert_name + " not updated.")
+
+    def get_server_cert(self, cert_name):
+        self.debug("getting server certificate: " + cert_name)
+        cert = self.euare.get_server_certificate(cert_name=cert_name)
+        self.debug(cert)
+        return cert
+
+    def delete_server_cert(self, cert_name):
+        self.debug("deleting server certificate: " + cert_name)
+        self.euare.delete_server_cert(cert_name)
+        if (cert_name) in str(self.euare.list_server_certs()):
+            raise Exception("certificate " + cert_name + " not deleted.")
+
+    def list_server_certs(self, path_prefix='/', marker=None, max_items=None):
+        self.debug("listing server certificates")
+        certs = self.euare.list_server_certs(path_prefix=path_prefix, marker=marker, max_items=max_items)
+        self.debug(certs)
+        return certs
+
+    def create_login_profile(self, user_name, password, delegate_account=None):
+        self.debug("Creating login profile for: " + user_name + " with password: " + password)
+        params = {'UserName': user_name,
+                  'Password': password}
+        if delegate_account:
+            params['DelegateAccount'] = delegate_account
+        self.euare.get_response('CreateLoginProfile', params, verb='POST')
\ No newline at end of file
