@@ -32,7 +32,7 @@
 #
 # Author: vic.iglesias@eucalyptus.com
 
-__version__ = '0.0.9'
+__version__ = '0.0.10'
 
 import re
 import os
@@ -115,6 +115,10 @@ class Eutester(object):
         return ec2_url.split(':')[1].split("/")[0]
 
     def parse_eucarc(self, field):
+        if self.credpath is None:
+            raise ValueError('Credpath has not been set yet. '
+                             'Please set credpath or provide '
+                             'configuration file')
         with open( self.credpath + "/eucarc") as eucarc:
             for line in eucarc.readlines():
                 if re.search(field, line):
@@ -299,6 +303,7 @@ class Eutester(object):
 
         @wraps(func)
         def methdecor(*func_args, **func_kwargs):
+            _args_dict = {} # If method has this kwarg populate with args here
             try:
                 defaults = func.func_defaults
                 kw_count = len(defaults or [])
@@ -311,6 +316,19 @@ class Eutester(object):
                 for kw_name in kw_names: 
                     kw_defaults[kw_name] = defaults[kw_names.index(kw_name)]
                 arg_string=''
+                # If the underlying method is using a special kwarg named
+                # '_args_dict' then provide all the args & kwargs it was
+                # called with in that dict for inspection with that method
+                if 'self' in var_names and len(func_args) <= 1:
+                    func_args_empty = True
+                else:
+                    func_args_empty = False
+                if (not func_args_empty or func_kwargs) and \
+                                '_args_dict' in kw_names:
+                    if not '_args_dict' in func_kwargs or \
+                            not func_kwargs['_args_dict']:
+                        func_kwargs['_args_dict'] = {'args':func_args,
+                                                     'kwargs':func_kwargs}
                 #iterate on func_args instead of arg_names to make sure we pull out self object if present
                 for count, arg in enumerate(func_args):
                     if count == 0 and var_names[0] == 'self': #and if hasattr(arg, func.func_name):
