@@ -158,7 +158,7 @@ class Net_Tests(EutesterTestCase):
                                 'or config_file and password was provided, err:' + str(e))
                 #replace default eutester debugger with eutestcase's for more verbosity...
             self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
-        self.assertIsInstance(self.tester, Eucaops)
+        assert isinstance(self.tester, Eucaops)
         self.cc_last_checked = time.time()
 
         ### Create local zone list to run tests in
@@ -198,7 +198,7 @@ class Net_Tests(EutesterTestCase):
         if self.args.emi:
             self.image = self.tester.get_emi(emi=str(self.args.emi))
         else:
-            self.image = self.tester.get_emi(root_device_type="instance-store",not_location='windows')
+            self.image = self.tester.get_emi(root_device_type="instance-store")
         if not self.image:
             raise Exception('couldnt find instance store image')
 
@@ -211,7 +211,7 @@ class Net_Tests(EutesterTestCase):
 
     def authorize_group_for_instance_list(self, group, instances):
         for instance in instances:
-            self.assertIsInstance(instance, EuInstance)
+            assert isinstance(instance, EuInstance)
             self.tester.authorize_group(group, cidr_ip=instance.private_ip_address + "/32")
 
     def clean_method(self):
@@ -220,26 +220,34 @@ class Net_Tests(EutesterTestCase):
         else:
             self.tester.cleanup_artifacts()
 
-    def create_ssh_connection_to_instance_through_cc(self, instance, retry=10):
-        cc = self.get_active_cc_for_instance(instance)
+    def get_proxy_machine(self, instance):
+        if self.tester.config["network"].lower() == "edge":
+            proxy_machine = self.get_active_nc_for_instance(instance)
+        else:
+            proxy_machine = self.get_active_cc_for_instance(instance)
+        self.debug("Instance is running on: " + proxy_machine.hostname)
+        return proxy_machine
+
+    def create_ssh_connection_to_instance(self, instance, retry=10):
+        proxy_machine = self.get_proxy_machine(instance)
         ssh = None
         attempts = 0
         elapsed = 0
         next_retry_time = 10
         start = time.time()
-        proxy_keypath=cc.machine.ssh.keypath or None
+        proxy_keypath=proxy_machine.machine.ssh.keypath or None
         while not ssh and attempts < retry:
             attempts += 1
             elapsed = int(time.time()-start)
             self.debug('Attempting to ssh to instances private ip:' + str(instance.private_ip_address) +
-                       'through the cc ip:' + str(cc.hostname) + ', attempts:' +str(attempts) + "/" + str(retry) +
+                       'through the cc ip:' + str(proxy_machine.hostname) + ', attempts:' +str(attempts) + "/" + str(retry) +
                        ", elapsed:" + str(elapsed))
             try:
                 ssh = SshConnection(host=instance.private_ip_address,
                                 keypath=instance.keypath,
-                                proxy=cc.hostname,
-                                proxy_username=cc.machine.ssh.username,
-                                proxy_password=cc.machine.ssh.password,
+                                proxy=proxy_machine.hostname,
+                                proxy_username=proxy_machine.machine.ssh.username,
+                                proxy_password=proxy_machine.machine.ssh.password,
                                 proxy_keypath=proxy_keypath)
             except Exception, ce:
                 tb = self.tester.get_traceback()
@@ -251,7 +259,7 @@ class Net_Tests(EutesterTestCase):
 
         if not ssh:
             raise Exception('Could not ssh to instances private ip:' + str(instance.private_ip_address) +
-                            ' through the cc ip:' + str(cc.hostname) + ', attempts:' +str(attempts) + "/" + str(retry) +
+                            ' through the cc ip:' + str(proxy_machine.hostname) + ', attempts:' +str(attempts) + "/" + str(retry) +
                             ", elapsed:" + str(elapsed))
 
         return ssh
@@ -268,11 +276,15 @@ class Net_Tests(EutesterTestCase):
                                                                      state='ENABLED')[0]
         return cc
 
-    def ping_instance_private_ip_from_active_cc(self, instance, refresh_active_cc=30):
-        self.assertIsInstance(instance, EuInstance)
-        cc = self.get_active_cc_for_instance(instance=instance, refresh_active_cc=refresh_active_cc)
+    def get_active_nc_for_instance(self,instance):
+        nc = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id, use_cached_list=False).pop()
+        return nc
+
+    def ping_instance_private_ip_from_active_cc(self, instance):
+        assert isinstance(instance, EuInstance)
+        proxy_machine = self.get_proxy_machine(instance)
         try:
-            cc.machine.ping_check(instance.private_ip_address)
+            proxy_machine.machine.ping_check(instance.private_ip_address)
             return True
         except:pass
         return False
@@ -311,7 +323,7 @@ class Net_Tests(EutesterTestCase):
         for instance in self.group1_instances:
             self.status('Checking connectivity to:' + str(instance.id) + ":" + str(instance.private_ip_address)+
                         ", zone:" + str(instance.placement) )
-            self.assertIsInstance(instance, EuInstance)
+            assert isinstance(instance, EuInstance)
             self.debug('Attempting to ping instances private ip from cc...')
             self.tester.wait_for_result( self.ping_instance_private_ip_from_active_cc,
                                          result=True,
@@ -357,13 +369,13 @@ class Net_Tests(EutesterTestCase):
         for instance in self.group2_instances:
             self.status('Checking connectivity to:' + str(instance.id) + ":" + str(instance.private_ip_address)+
                         ", zone:" + str(instance.placement) )
-            self.assertIsInstance(instance, EuInstance)
+            assert isinstance(instance, EuInstance)
             self.tester.wait_for_result( self.ping_instance_private_ip_from_active_cc,
                                          result=True,
                                          timeout=ping_timeout,
                                          instance=instance)
             self.status('Make sure ssh is working through CC path before trying between instances...')
-            instance.cc_ssh = self.create_ssh_connection_to_instance_through_cc(instance)
+            instance.cc_ssh = self.create_ssh_connection_to_instance(instance)
             self.status('SSH connection to instance:' + str(instance.id) +
                         ' successful to private ip:' + str(instance.private_ip_address) +
                         ', zone:' + str(instance.placement))
@@ -399,7 +411,7 @@ class Net_Tests(EutesterTestCase):
             instance2 = None
             for instance in self.group1_instances:
                 if instance.placement == zone:
-                    self.assertIsInstance(instance, EuInstance)
+                    assert isinstance(instance, EuInstance)
                     instance1 = instance
                     break
             if not instance1:
@@ -407,7 +419,7 @@ class Net_Tests(EutesterTestCase):
 
             for instance in self.group2_instances:
                 if instance.placement == zone:
-                    self.assertIsInstance(instance, EuInstance)
+                    assert isinstance(instance, EuInstance)
                     instance2 = instance
                     break
             if not instance2:
@@ -425,7 +437,7 @@ class Net_Tests(EutesterTestCase):
         instance1.sys("ssh -o StrictHostKeyChecking=no -i "
                       + str(os.path.basename(instance1.keypath))
                       + " root@" + str(instance2.private_ip_address)
-                      + " ' uname -a'", code=0)
+                      + " 'uname -a'", code=0)
         self.debug('Ssh between instances passed')
 
     def test4_attempt_unauthorized_ssh_from_test_machine_to_group2(self):
@@ -435,7 +447,7 @@ class Net_Tests(EutesterTestCase):
         for ssh access from this source.
         '''
         for instance in self.group2_instances:
-            self.assertIsInstance(instance, EuInstance)
+            assert isinstance(instance, EuInstance)
             #Provide some debug information re this data connection in this security group
             self.tester.does_instance_sec_group_allow(instance=instance, src_addr=None, protocol='tcp',port=22)
             try:
@@ -473,7 +485,7 @@ class Net_Tests(EutesterTestCase):
             instance = None
             for instance in self.group1_instances:
                 if instance.placement == zone.zone:
-                    self.assertIsInstance(instance, EuInstance)
+                    assert isinstance(instance, EuInstance)
                     zone.test_instance_group1 = instance
                     break
                 instance = None
@@ -547,7 +559,7 @@ class Net_Tests(EutesterTestCase):
             instance = None
             for instance in self.group1_instances:
                 if instance.placement == zone.zone:
-                    self.assertIsInstance(instance, EuInstance)
+                    assert isinstance(instance, EuInstance)
                     zone.test_instance_group1 = instance
                     break
                 instance = None
@@ -556,7 +568,7 @@ class Net_Tests(EutesterTestCase):
             instance = None
             for instance in self.group2_instances:
                 if instance.placement == zone.zone:
-                    self.assertIsInstance(instance, EuInstance)
+                    assert isinstance(instance, EuInstance)
                     zone.test_instance_group2 = instance
                     break
             if not zone.test_instance_group2:
