@@ -30,6 +30,7 @@ Cleanup:
 -remove all volumes, instance, and snapshots created during this test
 
 '''
+import copy
 import types
 import time
 import os
@@ -51,95 +52,116 @@ class TestZone():
         return self.name
 
 class EbsTestSuite(EutesterTestCase):
-    
-    tester = None
-    zonelist = []
-    snaps = []
-    keypair = None
-    group = None
-    multicluster=False
-    image = None
-    
+
     def __init__(self, 
-                 name=None,
-                 args=None,
-                 tester=None, 
-                 zone=None, 
-                 config_file='../input/2b_tested.lst', 
-                 password="foobar",
-                 user_data=None,
-                 inst_pass=None,
-                 credpath=None, 
-                 volumes=None, 
-                 keypair=None, 
-                 group=None, 
-                 emi=None,
-                 waitconnect=30,
-                 wait_on_progress=20,
-                 root_device_type='instance-store',
-                 vmtype='c1.medium',
-                 eof=1):
-        
-        self.args = args
-        self.setuptestcase(name)
-        self.wait_on_progress = wait_on_progress
-        if tester is None:
-            self.tester = Eucaops( config_file=config_file,password=password,credpath=credpath)
+                 name='ebs test suite',
+                 tester=None,
+                 **kwargs):
+
+        self.setuptestcase(name=name)
+        self.setup_parser(testname='ebs test suite',
+                    description="Attempts to test and provide info on focused areas related to Eucalyptus EBS related functionality.",
+                    testlist=False)
+        self.parser.add_argument('--reboot_timeout',dest='waitconnect',
+                                 help='Time to wait before trying to connect to '
+                                      'rebooted guests',
+                                 default=30)
+        self.parser.add_argument('--group',
+                                 dest='group',
+                                 help='Security group to use in test',
+                                 default=None)
+        self.parser.add_argument('--waitconnect',
+                                 dest='waitconnect',
+                                 help='Time to wait before attempting to '
+                                      'connect to a rebooted instance',
+                                 default=30)
+        self.parser.add_argument('--wait_on_progress',
+                                 dest='wait_on_progress',
+                                 help='Time to wait for incremental '
+                                      'progress during snapshot creation',
+                                 default=20)
+        self.parser.add_argument('--root_device_type',dest='root_device_type',
+                                 help='Root device type for instance used '
+                                      'in test. Default: isntance-store',
+                                 default='instance-store')
+        self.parser.add_argument('--instance_password',
+                                 help="Instance password for ssh session if not "
+                                      "key enabled", default=None)
+        self.parser.add_argument('--no_clean_on_exit', dest='clean_on_exit',
+                                 help="Clean up resources created in this test. "
+                                      "Default=True",
+                                 action='store_false', default=True)
+        self.parser.add_argument('--exit_on_failure', dest='exit_on_failure',
+                                 help="End test on first test unit failure "
+                                    "Default=False",
+                                 action='store_true', default=False)
+        self.parser.add_argument('--volumes', dest='volumes',
+                                 help="Comma separated list of volumes to use "
+                                      "in this test",
+                                 default="")
+        self.get_args()
+             # Allow __init__ to get args from __init__'s kwargs or through command line parser...
+        for kw in kwargs:
+            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
+            self.set_arg(kw ,kwargs[kw])
+        self.show_args()
+        for key in self.args.__dict__:
+            if key != 'args':
+                print 'Setting local value:' + str(key) + ", value:" + str(self.args.__dict__[key])
+                setattr(self,key, self.args.__dict__[key])
+
         else:
-            self.tester = tester
-        self.tester.exit_on_fail = eof
-    
-        self.testlist =[]
-        self.inst_pass=inst_pass
-        if emi:
-            self.image = self.tester.get_emi(emi=emi)
-        else:
-            self.image = self.tester.get_emi(root_device_type=root_device_type, not_location='windows')
-        self.vmtype = vmtype
-        self.zone = None
-        self.waitconnect=int(waitconnect)
-        self.zonelist = []
-        self.user_data = user_data
-        #create some zone objects and append them to the zonelist
-        if self.zone:
-            self.zone = TestZone(zone)
-            self.zonelist.append(self.zone)
-        else: 
-            self.setup_testzones()
-    
-        #If the list of volumes passed in looks good, sort them into the zones
-        if self.volumes_list_check(volumes):
-            self.sort_volumes(volumes)
-            
-        #Setup our security group for later use
-        if (group is not None):
-            self.group = group
-        else:
-            group_name='EbsTestGroup'
-            
-            try:
-                self.group = self.tester.add_group(group_name,fail_if_exists=False)
-                self.tester.authorize_group_by_name(self.group.name)
-                self.tester.authorize_group_by_name(self.group.name,protocol="icmp",port=-1)
-            except Exception, e:  
-                self.debug(self.tester.get_traceback())  
-                raise Exception("Error when setting up group:"+str(group_name)+", Error:"+str(e))   
-        
-    
+            self.show_args()
+            self.multicluster = False
+            self.zonelist = []
+            self.snaps = []
+            if tester is None:
+                self.tester = Eucaops( config_file=self.configfile,password=self.password,credpath=self.credpath)
+            else:
+                self.tester = tester
+            if self.emi:
+                self.image = self.tester.get_emi(emi=self.emi)
+            else:
+                self.image = self.tester.get_emi(root_device_type=self.root_device_type)
+            #create some zone objects and append them to the zonelist
+            if self.zone:
+                self.zone = TestZone(self.zone)
+                self.zonelist.append(self.zone)
+            else:
+                self.setup_testzones()
+
+            #If the list of volumes passed in looks good, sort them into the zones
+            if self.volumes_list_check(self.volumes):
+                self.sort_volumes(self.volumes)
+
+            #Setup our security group for later use
+            if self.group:
+                if isinstance(self.group, types.StringType):
+                    self.group = self.tester.add_group(self.group)
+            else:
+                group_name='EbsTestGroup'
+                try:
+                    self.group = self.tester.add_group(group_name,fail_if_exists=False)
+                    self.tester.authorize_group_by_name(self.group.name)
+                    self.tester.authorize_group_by_name(self.group.name,protocol="icmp",port=-1)
+                except Exception, e:
+                    self.debug(self.tester.get_traceback())
+                    raise Exception("Error when setting up group:"+str(group_name)+", Error:"+str(e))
+
         #Setup the keypairs for later use
-        if not self.inst_pass:
+        if not self.instance_password:
             try:
-                if (keypair is not None):
-                    self.keypair = keypair
-                else:     
+                if not self.keypair:
                     keys = self.tester.get_all_current_local_keys() 
-                    if keys != []:
+                    if keys:
                         self.keypair = keys[0]
                     else:
                         self.keypair = keypair = self.tester.add_keypair('ebs_test_key-' + str(time.time()))
             except Exception, ke:
                 raise Exception("Failed to find/create a keypair, error:" + str(ke))
-        
+        print "###########################################################"
+        print "Done with EBS SUITE INIT"
+        print "###########################################################"
         
     def setup_testzones(self):
         for zone in self.tester.get_zones():
@@ -185,13 +207,15 @@ class EbsTestSuite(EutesterTestCase):
         for testzone in zonelist:
             zone = testzone.name
             vols = self.tester.create_volumes(zone, size=size, count=volsperzone, snapshot=snapshot,timepergig=timepergig)
+            for vol in vols:
+                vol.add_tag('ebstestsuite_created')
             testzone.volumes.extend(vols)
             self.debug('create_vols_per_zone created vols('+str(len(vols))+') zone:'+str(zone))
             
            
             
           
-    def create_test_instances_for_zones(self, zonelist=None, image=None, keypair=None, username='root', inst_pass=None, group=None, vmtype=None,count=1):
+    def create_test_instances_for_zones(self, zonelist=None, image=None, keypair=None, username='root', instance_password=None, group=None, vmtype=None,count=1):
         """
         Description:
                     Create an instance within each TestZone object in zonelist to help test ebs functionality.
@@ -208,7 +232,7 @@ class EbsTestSuite(EutesterTestCase):
             group = self.group
         if keypair is None:
             keypair = self.keypair
-        inst_pass = inst_pass or self.inst_pass
+        instance_password = instance_password or self.instance_password
                 
         vmtype = vmtype or self.vmtype
         if keypair:
@@ -222,7 +246,7 @@ class EbsTestSuite(EutesterTestCase):
                                                 keypair=keyname,
                                                 group=group,
                                                 username=username,
-                                                password=inst_pass,
+                                                password=instance_password,
                                                 user_data=self.user_data,
                                                 type=vmtype,
                                                 zone=zone,
@@ -234,7 +258,7 @@ class EbsTestSuite(EutesterTestCase):
             self.debug('Created instance: ' + str(inst.id)+" in zone:"+str(zone))
         #self.endsuccess()
     
-    def terminate_test_instances_for_zones(self, zonelist=None, timeout=360):
+    def terminate_test_instances_for_zones(self, zonelist=None, timeout=480):
         if zonelist is None:
             zonelist = self.zonelist
         for zone in zonelist:
@@ -242,7 +266,7 @@ class EbsTestSuite(EutesterTestCase):
                 self.tester.terminate_single_instance(instance, timeout)
                 zone.instances.remove(instance)
                 
-    def terminate_instances_in_zones_verify_volume_detach(self,zonelist=None,timeout=360):
+    def terminate_instances_in_zones_verify_volume_detach(self,zonelist=None,timeout=480):
         """
         Description:
                   Iterates over all instances in this testcase's zonelist attempts to terminate the instances,
@@ -257,7 +281,7 @@ class EbsTestSuite(EutesterTestCase):
                 instance.terminate_and_verify(verify_vols=True,timeout=timeout)
                 zone.instances.remove(instance)
 
-    def negative_attach_in_use_volume_in_zones(self,zonelist=None,timeout=360):
+    def negative_attach_in_use_volume_in_zones(self,zonelist=None,timeout=480):
         """
         Description:
                     Iterates though zones and attempts to attach already attached volumes to instances within each zone.  
@@ -288,7 +312,7 @@ class EbsTestSuite(EutesterTestCase):
                 raise Exception("No attached volumes found to test against")
                 
     
-    def attach_all_avail_vols_to_instances_in_zones(self, zonelist=None, timeout=360, overwrite=False):
+    def attach_all_avail_vols_to_instances_in_zones(self, zonelist=None, timeout=480, overwrite=False):
         """
         Description:
                     Iterates though zones and attempts to attach volumes to an instance within each zone.  
@@ -374,7 +398,7 @@ class EbsTestSuite(EutesterTestCase):
                             raise Exception("Was able to delete attached volume:"+str(volume.id))
                    
                         
-    def reboot_instances_in_zone_verify_volumes(self,zonelist=None,waitconnect=30, timeout=360):
+    def reboot_instances_in_zone_verify_volumes(self,zonelist=None,waitconnect=30, timeout=480):
         """
         Description:
                     Attempts to iterate through each instance in each zone and reboot the instance(s). 
@@ -391,7 +415,7 @@ class EbsTestSuite(EutesterTestCase):
                 instance.reboot_instance_and_verify(waitconnect=waitconnect, timeout=timeout, checkvolstatus=True)
         #self.endsuccess()
         
-    def detach_volumes_in_zones(self,zonelist=None, timeout=360, volcount=1, eof=False):
+    def detach_volumes_in_zones(self,zonelist=None, timeout=480, volcount=1, eof=False):
         """
         Description:
                     Attempts to detach volcount volumes from each instance in the provided zonelist. 
@@ -434,7 +458,7 @@ class EbsTestSuite(EutesterTestCase):
         if errmsg:
             raise Exception(errmsg)
         
-    def detach_all_volumes_from_stopped_instances_in_zones(self,zonelist=None, timeout=360):
+    def detach_all_volumes_from_stopped_instances_in_zones(self,zonelist=None, timeout=480):
         """
         Description:
                     Attempts to detach volumes from instances while in the stopped state and
@@ -520,7 +544,9 @@ class EbsTestSuite(EutesterTestCase):
             for volume in zone.volumes:
                 volume.update()
                 if volstate == "all" or volume.status == volstate:
-                    self.snaps.append(self.tester.create_snapshot_from_volume(volume, description="ebstest", wait_on_progress=wait_on_progress))
+                    new_snap = self.tester.create_snapshot_from_volume(volume, description="ebstest", wait_on_progress=wait_on_progress)
+                    new_snap.add_tag('ebstestsuite_created')
+                    self.snaps.append(new_snap)
         #self.endsuccess()
         
         
@@ -543,6 +569,7 @@ class EbsTestSuite(EutesterTestCase):
             for snap in zonesnaps:
                 self.debug("Creating volume from snap:"+str(snap.id))
                 newvol = self.tester.create_volume(zone.name, size=0, snapshot=snap,timepergig=timepergig)
+                newvol.add_tag('ebstestsuite_created')
                 zone.volumes.append(newvol)
                 snap.eutest_volumes.append(newvol)
         #self.endsuccess()
@@ -556,7 +583,7 @@ class EbsTestSuite(EutesterTestCase):
                 retlist.append(snap)
         return retlist
         
-    def attach_new_vols_from_snap_verify_md5(self,zonelist=None, timeout=360,timepergig=360):
+    def attach_new_vols_from_snap_verify_md5(self,zonelist=None, timeout=480,timepergig=480):
         """
         Description:
                     Attempts to attach volumes which were created from snapshots and are not in use. 
@@ -622,6 +649,7 @@ class EbsTestSuite(EutesterTestCase):
             for snap in self.snaps:
                 if snap.eutest_volume_zone != zone:
                     newvol = self.tester.create_volume(zone.name,size=0, snapshot=snap, timepergig=timepergig)
+                    newvol.add_tag('ebstestsuite_created')
                     zone.volumes.append(newvol)
                     snap.eutest_volumes.append(newvol)
         #self.endsuccess()
@@ -635,7 +663,7 @@ class EbsTestSuite(EutesterTestCase):
                                                 tpg=300,
                                                 delete_to=120,
                                                 poll_progress=60,
-                                                attach_timeout=360):
+                                                attach_timeout=480):
         """
         Description:
                    Attempts to create a 'count' number of snapshots consecutively with a delay of 'delay'
@@ -667,7 +695,10 @@ class EbsTestSuite(EutesterTestCase):
             self.debug('Finished creating '+str(count)+' snapshots in zone:'+str(zone.name)+', now creating vols from them')
             try:
                 for snap in snaps:
-                    createdvols.extend(self.tester.create_volumes(zone,snapshot=snap,timepergig=tpg, monitor_to_state=False))
+                    new_vols = self.tester.create_volumes(zone,snapshot=snap,timepergig=tpg, monitor_to_state=False)
+                    for vol in new_vols:
+                        vol.add_tag('ebstestsuite_created')
+                    createdvols.extend(new_vols)
                 vols.extend(self.tester.monitor_created_euvolumes_to_state(createdvols, timepergig=tpg))
                 self.tester.print_euvolume_list(vols)
                 self.status("Attempting to attach new vols from new snapshots to instance:"+str(instance.id)+" to verify md5s...")
@@ -715,7 +746,7 @@ class EbsTestSuite(EutesterTestCase):
                                                             tpg=300,
                                                             delete_to=120,
                                                             poll_progress=60,
-                                                            attach_timeout=360):
+                                                            attach_timeout=480):
         """
         Description:
                    Attempts to create a 'count' number of volumes from a given snapshot consecutively with a delay of 'delay'
@@ -748,7 +779,10 @@ class EbsTestSuite(EutesterTestCase):
             for zone in zonelist:
                 self.status('STARTING ZONE:'+str(zone.name))
                 #Do not set monitor flag in order to quickly request count number of consecutive vols in each zone
-                vols.extend(self.tester.create_volumes(zone,snapshot=snap, count=count, monitor_to_state=None, timepergig=tpg))
+                new_vols = self.tester.create_volumes(zone,snapshot=snap, count=count, monitor_to_state=None, timepergig=tpg)
+                for vol in new_vols:
+                    vol.add_tag('ebstestsuite_created')
+                vols.extend(new_vols)
             vols = self.tester.monitor_created_euvolumes_to_state(vols,timepergig=tpg)
             self.tester.print_euvolume_list(vols)
             for zone in zonelist:
@@ -865,13 +899,17 @@ class EbsTestSuite(EutesterTestCase):
             raise Exception("Zone list was empty")
         for testzone in zonelist:
             vols = self.tester.create_volumes(testzone, size=size, count=volsperzone)
+            for vol in vols:
+                vol.add_tag('ebstestsuite_created')
             testzone.volumes.extend(vols)
             snapshots = []
             for volume in vols:
                 snapshots.append(self.tester.create_snapshot_from_volume(volume))
             larger_volumes = []
             for snaphot in snapshots:
-                larger_volumes.append(self.tester.create_volume(testzone, snapshot=snaphot, size=size+1))
+                larger_volume = self.tester.create_volume(testzone, snapshot=snaphot, size=size+1)
+                larger_volume.add_tag('ebstestsuite_created')
+                larger_volumes.append(larger_volume)
             for volume in larger_volumes:
                 assert volume.size > size
 
@@ -935,7 +973,7 @@ class EbsTestSuite(EutesterTestCase):
             start = time.time()
             elapsed = 0
             sc = None
-            while (sc is None) and (elapsed < 360):
+            while (sc is None) and (elapsed < 480):
                 self.debug("waiting for sc in zone:"+str(zone.name)+" elapsed:"+str(elapsed))
                 elapsed = int(time.time()-start)
                 sc = zone.partition.get_enabled_sc()
@@ -977,15 +1015,16 @@ class EbsTestSuite(EutesterTestCase):
     def test_max_volume_size_property(self, volumes=None, maxsize=1, zones=None):
         if zones is None or zones == []:
             zones = self.zones
+
     def clean_method(self):
         """
         Definition:
         Attempts to clean up test artifacts created during this test
         """
 
-        self.clean_created_resources(zonelist=self.zonelist, timeout=360)
+        self.clean_created_resources(zonelist=self.zonelist, timeout=480)
     
-    def clean_created_resources(self, zonelist=None, timeout=360):
+    def clean_created_resources(self, zonelist=None, timeout=480):
         """
         Definition:
         Attempts to clean up test artifacts created during this test
@@ -995,47 +1034,19 @@ class EbsTestSuite(EutesterTestCase):
         #self.delete_volumes_in_zones(zonelist=zonelist, timeout=timeout)
         #self.delete_snapshots_in_zones(zonelist=zonelist,  timeout=timeout)
    
-            
-    
 if __name__ == "__main__":
     ## If given command line arguments, use them as test names to launch
+    testcase= EbsTestSuite()
+    testcase.clean_method = testcase.clean_created_resources
+    testlist = testcase.ebs_basic_test_suite(run=False)
+
+    ret = testcase.run_test_case_list(testlist,
+                                      eof=testcase.args.exit_on_failure,
+                                      clean_on_exit=testcase.args.clean_on_exit)
+    print "ebs_basic_test exiting:("+str(ret)+")"
+    exit(ret)
 
-    tc = EutesterTestCase()
 
-    tc.setup_parser(testname='ebstestsuite.py', description='collection of ebs related tests', testlist=False)
-    tc.parser.add_argument('--inst_pass', 
-                        help="Instance password for ssh session if not key enabled", default=None)
-    
-    args = tc.get_args()
-    #if file was not provided or is not found
-    if not os.path.exists(args.config):
-        print "Error: Mandatory Config File '"+str(args.config)+"' not found."
-        tc.parser.print_help()
-        exit(1)
-    #ebssuite = EbsTestSuite(zone=args.zone, config_file= args.config, password=args.password,credpath=args.credpath, keypair=args.keypair, group=args.group, image=args.emi)
-    ebssuite = tc.do_with_args(EbsTestSuite)
-    kbtime=time.time()
-    try:
-       list = ebssuite.ebs_basic_test_suite(run=False)
-       tc.run_test_case_list(list)
-    except KeyboardInterrupt:
-        ebssuite.debug("Caught keyboard interrupt...")
-        if ((time.time()-kbtime) < 2):
-            ebssuite.clean_created_resources()
-            ebssuite.debug("Caught 2 keyboard interupts within 2 seconds, exiting test")
-            ebssuite.clean_created_resources()
-            tc.print_test_list_results(list)
-            raise
-        else:          
-            tc.print_test_list_results(list)
-            kbtime=time.time()
-            pass     
-    except Exception, e:
-        raise e
-        exit(1)
-    finally:
-        tc.print_test_list_results(list)
-    exit(0)
         
 
 
