@@ -66,7 +66,7 @@ class Qa_214_volume_churn(EutesterTestCase):
             raise Exception("Failed to find/create a keypair, error:" + str(ke))
        
         ### Get an image to work with
-        self.image = self.tester.get_emi(root_device_type="instance-store",not_location='windows') 
+        self.image = self.tester.get_emi(root_device_type="instance-store")
         if not self.image:
             raise Exception('couldnt find image')
         self.clean_method = self.cleanup
